unit Unit1;

{$mode objfpc}{$H+}
{$define TRY_DOWNLOAD}
interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs,
  LCLIntf, LCLType, ExtCtrls, Menus, ComCtrls,
  X3DNodes, X3DLoad,
  {$ifndef VER3_0} OpenSSLSockets, {$endif}
  CastleDownload, CastleParameters, CastleClassUtils, CastleImages,
  CastleControl, CastleSceneCore, CastleVectors, CastleScene, CastleViewport,
  CastleNotifications, CastleTransform,
  // CastleUIControls, CastleLCLUtils,
  CastleRectangles, CastleKeysMouse, CastleLog, CastleTimeUtils;

type

  { TForm1 }

  TForm1 = class(TForm)
    CastleControlBase1: TCastleControlBase;
    Panel1: TPanel;
    procedure CastleControlBase1BeforeRender(Sender: TObject);
    procedure CastleControlBase1Press(Sender: TObject; const Event: TInputPressRelease);
    procedure CastleControlBase1Update(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ToggleFullScreen;
  private
    FullScreen: Boolean;
    Viewport: TCastleViewport;
    Scene: TCastleScene;
    stateNotifications: TCastleNotifications;
    infoNotifications: TCastleNotifications;
    screenRect: TFloatRectangle;
    frame: Integer;
    start_ticks: Int64;
    procedure LoadScene(filename: String);
    procedure InfoPanel;
  public

  end;

var
  Form1: TForm1;

const
  BorderNone = Controls.bsNone;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.InfoPanel;
var
  msg: String;
begin
  screenRect := Viewport.RenderRect;
  if screenRect.IsEmpty then
    msg := 'ScreenRect = Undefined'
  else
    msg := 'ScreenRect = MinX : ' + FloatToStr(screenRect.Left) + ', ' +
           'MinY : ' + FloatToStr(screenRect.Bottom) + ', ' +
           'Wide : ' + FloatToStr(screenRect.Width) + ', ' +
           'Tall : ' + FloatToStr(screenRect.Height);

  Panel1.Caption := msg;

  WritelnLog(msg);
end;

procedure TForm1.ToggleFullScreen;
begin
  FullScreen := not FullScreen;

  if FullScreen then
    begin
      {$ifdef unix}
      ShowWindow(Handle, SW_SHOWFULLSCREEN);
      {$else}
      BorderStyle := BorderNone;
      WindowState := wsFullScreen;
      FormStyle := fsSystemStayOnTop;
      {$endif}
    end
  else
    begin
      {$ifdef unix}
      ShowWindow(Handle, SW_SHOWNORMAL);
      {$else}
      WindowState := wsNormal; // wsMinimized, wsMaximized, wsFullScreen
      BorderStyle := bsSizeable; // bsDialog, bsNone, bsSingle
      FormStyle := fsNormal;
      ShowInTaskBar := stDefault; // stAlways, stNever
      {$endif}
    end;

    InfoPanel;
end;

procedure TForm1.CastleControlBase1Press(Sender: TObject;
  const Event: TInputPressRelease);
var
  TopMostScene: TCastleTransform;
begin
  if Event.IsKey(keyF11) then
    begin
      ToggleFullScreen;
    end;

  if not (stateNotifications = nil) then
  begin
    if Event.IsMouseButton(mbLeft) then
    begin
      if Viewport.MouseRayHit <> nil then
        TopMostScene := Viewport.MouseRayHit.First.Item
      else
        TopMostScene := nil;

      if TopMostScene <> nil then
        stateNotifications.Show('Clicked on scene ' + IntToStr(TopMostScene.Tag))
      else
        stateNotifications.Show('Clicked on empty space')
    end;
  end;
end;

procedure TForm1.CastleControlBase1BeforeRender(Sender: TObject);
var
  ZPlane: Single;
  CPos, CView, CUp, Projection: TVector3;
  msg: String;
begin
  ZPlane := -1;

  if Viewport.PositionToWorldPlane(Vector2(128, 72), True, Zplane, Projection) then
    msg := 'X : ' + FormatFloat('####0.00', Projection.X) +
           ', Y : ' + FormatFloat('####0.00', Projection.Y) +
           ', Z : ' + FormatFloat('####0.00', Projection.Z)
  else
    msg := 'Not possible';

  infoNotifications.Show(msg);
  WritelnLog(msg);

  Viewport.Camera.GetView(CPos, CView, CUp);

  msg := 'Position = X : ' + FormatFloat('####0.00', CPos.X) +
       ', Y : ' + FormatFloat('####0.00', CPos.Y) +
       ', Z : ' + FormatFloat('####0.00', CPos.Z) + ' | ' +
       'Direction = X : ' + FormatFloat('####0.00', CView.X) +
       ', Y : ' + FormatFloat('####0.00', CView.Y) +
       ', Z : ' + FormatFloat('####0.00', CView.Z);

  stateNotifications.Show(msg);
  WritelnLog(msg);
end;

procedure TForm1.CastleControlBase1Update(Sender: TObject);
begin
//  WritelnLog('Update time : '+ IntToStr(CastleGetTickCount64 - start_ticks));
  InfoPanel;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  Application.ProcessMessages;
end;

procedure TForm1.LoadScene(filename: String);
begin
  if not (Scene = nil) then
    begin
      Scene.Free;
      Scene := nil;
    end;

  CastleControlBase1.Enabled:=False;

  Viewport := TCastleViewport.Create(Application);
  Viewport.FullSize := true;
  Viewport.AutoCamera := true; // instead of this, you could do "Viewport.Camera.SetView(...)"
  Viewport.AutoNavigation := true; // instead of this, you could do "Viewport.Navigation := ..."

  stateNotifications := TCastleNotifications.Create(Application);
  stateNotifications.Anchor(hpLeft, 10);
  stateNotifications.Anchor(vpBottom, 10);

  infoNotifications := TCastleNotifications.Create(Application);
  infoNotifications.Anchor(hpRight, -10);
  infoNotifications.Anchor(vpBottom, 10);

  CastleControlBase1.Controls.InsertFront(Viewport);
  CastleControlBase1.Controls.InsertFront(stateNotifications);
  CastleControlBase1.Controls.InsertFront(infoNotifications);

  Scene := TCastleScene.Create(Application);
  Scene.Spatial := [ssRendering, ssDynamicCollisions];
  Scene.ProcessEvents := true;

  Scene.Load(filename);
  Scene.Tag := 1;
  Viewport.Items.Add(Scene);

  Viewport.Items.MainScene := Scene;

  CastleControlBase1.Enabled:=True;

  InfoPanel;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FullScreen := False;
  Form1.Width := 1280;
  Form1.Height := 800;
  Form1.Caption := 'view3d';
  frame := 0;
  start_ticks := CastleGetTickCount64;
  InitializeLog;
  LoadScene('castle-data:/table.x3dv');
end;

end.

